/***************************************************************************
 *
 *   ::::_audioeditor_::::
 *   a simple multitrack audio editor
 *
 *   $FILENAME$
 *   header file
 *
 *   begin                : Jan 2006
 *   copyright            : (C) 2006 by Georg Holzmann
 *   email                : grh@mur.at
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

#include <QtGui>
#include "mainwindow.h"
#include "audioctl.h"
#include "trackmanager.h"

MainWindow::MainWindow()
{
  m_trackmanager = new TrackManager;
  setCentralWidget(m_trackmanager);

  createActions();
  createMenus();
  createToolBars();
  createStatusBar();

  readSettings();
  setCurrentFile("");
  
  m_audioctl = new AudioCtl;
  m_stopped=true;
  
  connect(m_trackmanager->document(), SIGNAL(contentsChanged()),
          this, SLOT(documentWasModified()));
}

MainWindow::~MainWindow()
{
  delete m_audioctl;
  delete m_trackmanager;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (maybeSave()) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

void MainWindow::newFile()
{
  if (maybeSave()) {
    m_trackmanager->clear();
    setCurrentFile("");
  }
}

void MainWindow::open()
{
  QMessageBox::warning(this, "TODO", "implement this");
  /*
  if (maybeSave()) {
    QString fileName = QFileDialog::getOpenFileName(this);
    if (!fileName.isEmpty())
      loadFile(fileName);
  }
  */
}

bool MainWindow::save()
{
  QMessageBox::warning(this, "TODO", "implement this");
  return false;
  
  /*
  if (m_curFile.isEmpty()) {
    return saveAs();
  } else {
    return saveFile(m_curFile);
  }
  */
}

bool MainWindow::saveAs()
{
  QMessageBox::warning(this, "TODO", "implement this");
  return false;
  
  /*
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
                                                    m_curFile);
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
  */
}

void MainWindow::about()
{
   QMessageBox::about(this, tr("About audioeditor"),
            tr( "<b>audioeditor</b>" 
                "<br><br> a simple open-source multitrack audio-editor"
                "<br><br> (C) 2006 by Georg Holzmann, grh@mur.at"));
}

void MainWindow::documentWasModified()
{
    setWindowModified(true);
}

void MainWindow::playSlot()
{
  if(m_stopped)
  {
    m_stopped=false;
    playAct->setIcon(QIcon(":/images/pause.xpm"));
    statusBar()->showMessage(tr("Starting playback"), 2000);
    m_audioctl->play();
  }
  else
  {
    if(playAct->isChecked ())
    {
      playAct->setIcon(QIcon(":/images/pause.xpm"));
      statusBar()->showMessage(tr("Continue playback"), 2000);
    }
    else
    {
      playAct->setIcon(QIcon(":/images/play.xpm"));
      statusBar()->showMessage(tr("Pause playback"), 2000);
    }
    
    m_audioctl->pause();
  }
}

void MainWindow::stopSlot()
{
  m_stopped = true;
  playAct->setChecked (false);
  playAct->setIcon(QIcon(":/images/play.xpm"));
  statusBar()->showMessage(tr("Stop playback"), 2000);
  m_audioctl->stop();
}

void MainWindow::createActions()
{
    newAct = new QAction(QIcon(":/images/new.png"), tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the document to disk"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), this);
    saveAsAct->setStatusTip(tr("Save the document under a new name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

    cutAct = new QAction(QIcon(":/images/cut.png"), tr("Cu&t"), this);
    cutAct->setShortcut(tr("Ctrl+X"));
    cutAct->setStatusTip(tr("Cut the current selection's contents to the "
                            "clipboard"));
    connect(cutAct, SIGNAL(triggered()), m_trackmanager, SLOT(cut()));

    copyAct = new QAction(QIcon(":/images/copy.png"), tr("&Copy"), this);
    copyAct->setShortcut(tr("Ctrl+C"));
    copyAct->setStatusTip(tr("Copy the current selection's contents to the "
                             "clipboard"));
    connect(copyAct, SIGNAL(triggered()), m_trackmanager, SLOT(copy()));

    pasteAct = new QAction(QIcon(":/images/paste.png"), tr("&Paste"), this);
    pasteAct->setShortcut(tr("Ctrl+V"));
    pasteAct->setStatusTip(tr("Paste the clipboard's contents into the current "
                              "selection"));
    connect(pasteAct, SIGNAL(triggered()), m_trackmanager, SLOT(paste()));
    
    
    playAct = new QAction(QIcon(":/images/play.xpm"), tr("Play"), this);
    //playAct->setShortcut(Qt::Key_Space);
    playAct->setShortcut(tr("Space"));
    playAct->setStatusTip(tr("Start/Pause playback"));
    connect(playAct, SIGNAL(triggered()), this, SLOT(playSlot()));
    playAct->setCheckable(true);
    
    stopAct = new QAction(QIcon(":/images/stop.xpm"), tr("Stop"), this);
    stopAct->setShortcut(tr("ESC"));
    stopAct->setStatusTip(tr("Stop playback"));
    connect(stopAct, SIGNAL(triggered()), this, SLOT(stopSlot()));
    
    
    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    cutAct->setEnabled(false);
    copyAct->setEnabled(false);
    connect(m_trackmanager, SIGNAL(copyAvailable(bool)),
            cutAct, SLOT(setEnabled(bool)));
    connect(m_trackmanager, SIGNAL(copyAvailable(bool)),
            copyAct, SLOT(setEnabled(bool)));
}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAct);
    fileMenu->addAction(openAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(cutAct);
    editMenu->addAction(copyAct);
    editMenu->addAction(pasteAct);
    
    menuBar()->addSeparator();
    
    playbackMenu = menuBar()->addMenu(tr("&Playback"));
    playbackMenu->addAction(playAct);
    playbackMenu->addAction(stopAct);

    menuBar()->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
}

void MainWindow::createToolBars()
{
    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(newAct);
    fileToolBar->addAction(openAct);
    fileToolBar->addAction(saveAct);

    editToolBar = addToolBar(tr("Edit"));
    editToolBar->addAction(cutAct);
    editToolBar->addAction(copyAct);
    editToolBar->addAction(pasteAct);
    
    playbackToolBar = addToolBar(tr("Playback"));
    editToolBar->addAction(playAct);
    editToolBar->addAction(stopAct);
}

void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void MainWindow::readSettings()
{
    QSettings settings("grh.mur.at", "audioeditor");
    QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
    QSize size = settings.value("size", QSize(400, 400)).toSize();
    move(pos);
    resize(size);
}

void MainWindow::writeSettings()
{
    QSettings settings("grh.mur.at", "audioeditor");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
}

bool MainWindow::maybeSave()
{
    if (m_trackmanager->document()->isModified()) {
        int ret = QMessageBox::warning(this, tr("audioeditor"),
                     tr("The document has been modified.\n"
                        "Do you want to save your changes?"),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;
}

void MainWindow::loadFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
      QMessageBox::warning(this, tr("audioeditor"),
                           tr("Cannot read file %1:\n%2.")
                               .arg(fileName)
                               .arg(file.errorString()));
      return;
    }

    QTextStream in(&file);
    QApplication::setOverrideCursor(Qt::WaitCursor);
    m_trackmanager->setPlainText(in.readAll());
    QApplication::restoreOverrideCursor();

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("File loaded"), 2000);
}

bool MainWindow::saveFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
      QMessageBox::warning(this, tr("audioeditor"),
                           tr("Cannot write file %1:\n%2.")
                               .arg(fileName)
                               .arg(file.errorString()));
      return false;
    }

    QTextStream out(&file);
    QApplication::setOverrideCursor(Qt::WaitCursor);
    out << m_trackmanager->toPlainText();
    QApplication::restoreOverrideCursor();

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("File saved"), 2000);
    return true;
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    m_curFile = fileName;
    m_trackmanager->document()->setModified(false);
    setWindowModified(false);

    QString shownName;
    if (m_curFile.isEmpty())
      shownName = "untitled.xml";
    else
      shownName = strippedName(m_curFile);

    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("audioeditor")));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
  return QFileInfo(fullFileName).fileName();
}
