/***************************************************************************
 *
 *   ::::_audioeditor_::::
 *   a simple multitrack audio editor
 *
 *   trackmanager
 *   header file
 *
 *   begin                : Jan 2006
 *   copyright            : (C) 2006 by Georg Holzmann
 *   email                : grh@mur.at
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

#ifndef TRACKMANAGER_H__
#define TRACKMANAGER_H__

#include <QObject>
#include <QList>
#include <QReadWriteLock>
#include "portaudio.h"

#include <QTextEdit> // TODO: remove !!!

class Track;
class AudioCtl;

/*!
 * \class TrackManager
 *
 * contains the single tracks, manages the display
 * of the waveforms etc.
 */
class TrackManager : public QTextEdit // testing
{
  Q_OBJECT
  
  public:
    TrackManager();
    virtual ~TrackManager();
    
  protected:
    /*! append a new track */
    static void appendTrack();
    
  private:
    /*!
     * \brief mixes the individual tracks
     * 
     * this will add all the soundfile-data etc. to the audio output
     * 
     * @param inputBuffer output sample buffer (size = bufsize * channels)
     * @param outputBuffer input sample buffer (size = bufsize * channels)
     * @param framesPerBuffer number of sample frames to be processed 
     *        by the callback
     * @param outTime time in samples when the buffer(s) processed by
     *        this callback will begin being played at the audio output
     * @param userData value of a user supplied pointer passed to 
     *        Pa_OpenStream(), for storing synthesis data etc.
     * 
     * @return normally 0; nonzero if you want to stop audio
     * 
     * \note the lock should be avoided here !!!
     *       (look AudioCtl for more details)
     */
     static int trackAudioCallback(void *inputBuffer, void *outputBuffer,
                                  unsigned long framesPerBuffer,
                                  PaTimestamp outTime, void *userData );
    
  protected:
    /*! list with all the tracks */
    static QList<Track *> m_tracklist;
    static QReadWriteLock m_track_lock;
    
    /*! the audio controller */
    static AudioCtl *m_audioctl;
};

#endif // TRACKMANAGER_H__
